import pyfie
import matplotlib.pyplot as plt

# 利便のため PyFIE 関数が返すエラーコードに応じて例外を発生させる機能を有効化
pyfie.ctrl.enable_f_err_exception(True)

# 処理対象画像の読み込み
image = pyfie.imread("captured.png")

# 2値画像格納用メモリの用意
imagebin = pyfie.fnFIE_img_root_alloc(pyfie.F_IMG_BIN, 1, image.width,
                                      image.height)

# 固定しきい値による２値化
err = pyfie.fnFIE_binarize(image, imagebin, 128)
# 画像オブジェクトのプロット
imagebin.imshow()
# プロットした画像の表示
pyfie.plot_show()
# imagebin画像を保存
pyfie.imwrite("imagebin.png", imagebin)

# ブローブ解析パラメータの設定
params = pyfie.F_MEASURE_PARAMS(
    0,  # 最大ラン数
    0,  # 最大ブローブ数
    0,  # 最大行数
    pyfie.F_MEASURE_BLACKFG,  # 対象色モード
    8,  # 連結モード
    0,  # 事前計算特徴量指定
    False  # 結果維持オプション
)

# 2値ブローブ解析の実行
err = pyfie.INT()
result = pyfie.fnFIE_measure_execute(imagebin, (0, 0), params, err.ref)

# フィルタ配列の設定
filter_num = 2
filters = pyfie.F_MEASURE_FILTER_RANGE.ARRAY(filter_num)
filters.value = (
    (pyfie.F_FEATURE_AREA, 100, 5000),
    (pyfie.F_FEATURE_XMIN, 150, 300)
)

# フィルタリングしたブローブ番号配列の取得
blob_numbers = pyfie.UINT.PTR()
blob_num = pyfie.UINT()
pyfie.fnFIE_measure_get_list(
    result, filters, filter_num, blob_numbers.ref, blob_num.ref
)

# 重心座標を受け取るための変数を用意
center_x = pyfie.DOUBLE()
center_y = pyfie.DOUBLE()
# 2値ブローブ解析結果の表示
for i in range(blob_num):
    # 重心の取得
    pyfie.fnFIE_measure_get_center(
        result, blob_numbers[i], center_x.ref, center_y.ref
    )
    print("重心 (", center_x, ",", center_y, ")")

# 画像オブジェクトのプロット
imagebin.imshow()
# 結果をプロット
pyfie.mpltools.plot_measure_results(result, blob_numbers, blob_num)
# 結果をプロットした画像の保存
plt.savefig("blob_result.png")
# 結果をプロットした画像の表示
pyfie.plot_show()

# メモリの解放、ブローブ番号配列を解放
blob_numbers.fnOAL_free()
